#!/bin/bash
#

#
# Type  : Utility
#
# Author: Rob Allen
#
# Date  : 09.04.2008
#
# Program Description
# ===================
# Backup script to create a full backup of an Active/Active appliance 
# and PINsafe using a replicated MySQL DB.
#
#
# Amendment History
# =================
# 04.06.08 Included bash_history files in the backup (RA)
# 23.06.08 Added additional support for single appliance (RA)
# 29.06.08 Added fix to start MySQL prior to taking a DB backup (RA)
# 04.07.08 changed logic on appliance detection and backup of DBs, also detect
#          what services are running prior to a backup, and return to same state
# 06.07.08 Added facility to store backup comments (RA)
# 25.02.09 Added line to echo the backup file to latest.txt file (RA)
# 02.06.09 Added MYSQL routine to determine version (RA)
# 19.06.09 Updated backup routine and removed some prompts to simplify the
#          process. PINsafe backup does not backup MySQL mysql DB. (RA)
# 15.07.09 Bug fix, missed mysql lock file on the 2.6.09 update (RA)
# 17.07.09 Fixed bug in backup script (RA)
# 15.08.09 Added option for pinsafe.conf and cmi.conf (RA)
# 24.09.09 Updated check_service to verify the tomcat process has died, before
#          continuing. (RA)
# 24.09.09 Added sleeps to the backup run order, to allow time to stop and
#          start services (RA)
# 25.09.09 Changed screen display to reflect backup process (RA)
# 25.10.09 Changed mysql backup to auto detect the mysql db in use (RA)
# 14.12.09 Added check to backup eth2 if file exists (RA)
# 17.02.10 Included passwd and shadow in factory backup (RA)
# 18.03.10 Only create mysql directory, if it is being backup up. (RA)
# 09.08.10 Added routine to check on mysql run status, and return the process
#          back to its starting state after the backup (RA)
# 09.08.10 Removed network backup process (RA)
# 09.08.10 Changed backup process to use a single archive, and a .info file
# 06.12.10 Removed check on PINSAFE_TYPE from 'build_info_file' (RA)
# 13.12.10 Changed 'appliance_check' to check on Shipping/internal on A/A (RA)
# 23.12.10 Removed DB_IN_USE from the info file (RA)
# 23.12.10 Changed value of BACKUP_TYPE to "A" or "P"
# 07.02.11 Fixed a bug on Tomcat stop/start during backup.
#

dir_name=`date +%d%m%y.$$`
BACKUPROOT=/backups/$dir_name
PINSAFE_CONF=/etc/pinsafe.conf
NETWORKS=$BACKUPROOT/networking
CONFIG_XML=/usr/local/tomcat/webapps/pinsafe/WEB-INF/conf/config.xml
ALL="_all"; PIN="_pin"
LOCK=/var/lock/subsys
AUDIT_LOG=/var/log/cmi/audit.log
inv=`tput smso`
nor=`tput rmso`
backup_vers="0.13"
backup_reason="$2"

initialise() {
 PINSAFE_TYPE=`grep PINSAFE_TYPE $PINSAFE_CONF|cut -d\= -f2`
 if test ! $PINSAFE_TYPE; then
  stnd_err "Appliance type is not correctly configured."; fi
 if test "$PINSAFE_TYPE" = "Single"; then
  PINSAFE_TYPE="Single "; fi
 
 check_on_DR
 DR=`grep "^DR=" $PINSAFE_CONF|cut -d\= -f2`

 if test $PINSAFE_TYPE = "Primary" || test $PINSAFE_TYPE = "Standby"; then
  APPLIANCE_TYPE="AA"; fi

 if test "$PINSAFE_TYPE" = "Single "; then
  APPLIANCE_TYPE="SA"; fi

 if test -f /etc/init.d/mysqld; then
  MYSQL=mysqld
 else
  MYSQL=mysql
 fi
}

check_on_DR() {
 if test $PINSAFE_TYPE = "Primary" || test $PINSAFE_TYPE = "Standby"; then
  sed -i "s/DR=yes/DR=no/g" $PINSAFE_CONF; fi
}

stnd_err() {
 clear
 echo; echo "$1"
 echo; read -p "Press any key to exit" yn
 stty intr ^C
 exit
}

create_backup_area() {
 mkdir -p $BACKUPROOT
 mkdir -p /var/log/cmi
 if test ! -d $BACKUPROOT; then
  echo "Backup area has failed to create. Please contact your support representative"
  sleep 5
  exit; fi

 cd $BACKUPROOT
 mkdir $@
}

audit() {
 audit_date=`date "+%d%m%y %T"`
 echo "$audit_date : $1" >> $AUDIT_LOG
}

networking() {
 echo "Copying network configuration"
 cp -p --parents /etc/sysconfig/network-scripts/ifcfg-eth0 $BACKUPROOT
 if test -f /etc/sysconfig/network-scripts/ifcfg-eth1; then
  cp -p --parents /etc/sysconfig/network-scripts/ifcfg-eth1 $BACKUPROOT; fi
 if test -f /etc/sysconfig/network-scripts/ifcfg-eth2; then
  cp -p --parents /etc/sysconfig/network-scripts/ifcfg-eth2 $BACKUPROOT; fi
 cp -p --parents /etc/sysconfig/network $BACKUPROOT
 cp -p --parents /etc/hosts $BACKUPROOT
 cp -p --parents /etc/resolv.conf $BACKUPROOT
 cp -p --parents /etc/sysconfig/iptables $BACKUPROOT
 cp -p --parents /etc/sysconfig/iptables-config $BACKUPROOT
 cp -p --parents /etc/pinsafe.conf $BACKUPROOT
 if test -f /etc/pinsafe.conf; then
  cp -p --parents /etc/pinsafe.conf $BACKUPROOT; fi
 if test -f /etc/cmi.conf; then
  cp -p --parents /etc/cmi.conf $BACKUPROOT; fi
 netstat -nr > $NETWORKS/netstat.out
 ifconfig > $NETWORKS/ifconfig.out
}

misc() {
 echo "Copying Keystore, ssh, snmp and /etc/profile"
 if test "`ls /home/swivel/.keystore* 2>/dev/null`"; then
  cp -p /home/swivel/.keystore* $BACKUPROOT/keystore; fi
 cp -rp --parents /root/.ssh $BACKUPROOT
 if test -f /root/.bash_history; then
  cp -rp --parents /root/.bash_history $BACKUPROOT; fi
 if test -f /home/swivel/.bash_history; then
  cp -rp --parents /home/swivel/.bash_history $BACKUPROOT; fi
 if test -d /etc/snmp; then
  cp -rp --parents /etc/snmp $BACKUPROOT; fi
 cp -p --parents /etc/profile $BACKUPROOT
 cp -p --parents /etc/cron.daily/PINsafe_backup* $BACKUPROOT
 cp -p --parents /etc/cron.daily/timesync.sh $BACKUPROOT
 cp -p --parents /etc/passwd $BACKUPROOT
 cp -p --parents /etc/shadow $BACKUPROOT
}

check_running() {
 if test "`ps -eaf|grep $2|egrep -v "UID|grep|mysqld_safe" 2>/dev/null`"; then
  if test ! -f $LOCK/$1; then
   touch $LOCK/$1; fi
 else
  if test -f $LOCK/$1; then
   rm -f $LOCK/$1; fi
 fi

 if test -f $LOCK/$1; then
  value="Running"
 else
  value="Stopped"
 fi
}

service_sands() {
 service $1 $2
 audit "Service $1 : $2"
}

mysql() {
 if test $DB_IN_USE = "MySQL"; then
  mkdir -p $BACKUPROOT/mysql

  check_running $MYSQL mysqld
  if test $value = "Stopped"; then
   service_sands $MYSQL start 2>/dev/null; fi

  if test $BACKUP_TYPE = "A"; then
   echo "Copying MySQL: \"mysql\" DB"
   if test -f /etc/my.cnf; then
    cp -p --parents /etc/my.cnf $BACKUPROOT; fi
   mysqldump mysql > $BACKUPROOT/mysql/mysql_backup.sql
  fi
 
  if test $BACKUP_TYPE = "P" || test $BACKUP_TYPE = "A"; then
   DB_NAME_IN_USE="`grep jdbc:mysql: $CONFIG_XML | cut -d\/ -f4|cut -d\< -f1`"
   echo "Copying MySQL: \"$DB_NAME_IN_USE\" DB"
   mysqldump --single-transaction --flush-logs $DB_NAME_IN_USE > $BACKUPROOT/mysql/pinsafe_backup.sql
  fi

  if test $value = "Stopped"; then
   service_sands $MYSQL stop 2>/dev/null; fi
 else
  echo "External DB used: PINsafe DB skipped"
 fi
}

tomcat() {
 echo "Copying Tomcat webapps and log files"
 if test $1 = "normal"; then
  cp -rp /usr/local/tomcat/webapps $BACKUPROOT/tomcat; fi
 if test $1 = "mounted"; then
  mkdir -p $BACKUPROOT/tomcat/webapps
  cp -rp /mnt/drbd-tmp/* $BACKUPROOT/tomcat/webapps; fi

 if test -d /usr/local/tomcat/webapps2; then
  cp -rp /usr/local/tomcat/webapps2 $BACKUPROOT/tomcat; fi
 cp -rp /usr/local/tomcat/conf/server.xml $BACKUPROOT/tomcat
 cp -rp /var/log/tomcat $BACKUPROOT/tomcat/logs
 cp -p /etc/rc.d/init.d/tomcat5 $BACKUPROOT/tomcat
}

webmin() {
 echo "Copying Webmin"
 if test -d /etc/webmin; then
  cp -rp --parents /etc/webmin  $BACKUPROOT; fi
}

mon() {
 echo "Copying MON"
 cp -rp --parents /etc/mon  $BACKUPROOT
 cp -rp --parents /usr/lib/mon  $BACKUPROOT
}

heartbeat() {
 if test -d /etc/ha.d; then
  echo "Copying Heartbeat"
  cp -rp --parents /etc/ha.d  $BACKUPROOT
 fi
}

drbd() {
 echo "Copying DRBD"
 cp -p --parents /etc/drbd.conf $BACKUPROOT
}

scripts() {
 echo "Copying scripts"
 if test -d /usr/local/bin; then
  cp -rp --parents /usr/local/bin $BACKUPROOT; fi
}

backup_start() {
 if test $1; then
 echo "############################################################"
 echo "            ${inv}Creating $1 default backup${nor}"
 echo "############################################################"
 echo
 fi
 echo "############################################################"
 echo "       Backup Started : `date`"
 echo "############################################################"
 echo
 date > $BACKUPROOT/backup.date
}

backup_end() {
 echo
 echo "############################################################"
 echo "       Backup Finished : `date`"
 echo "############################################################"
}

db_used() {
 CONFIG_XML=/usr/local/tomcat/webapps/pinsafe/WEB-INF/conf/config.xml
 XML="<option generated=\"true\" selected=\"true\""

 DB_IN_USE=Shipping
 if test -f $CONFIG_XML; then
  if grep "$XML>Internal</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=Internal; fi
  if grep "$XML>MySQL 5</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=MySQL; fi
 else
  if test ! $APPLIANCE_TYPE = "AP"; then
   echo "Unable to find the config.xml file."; fi
 fi
}

sleep_disp() {
 count=1
 while test $count -lt $1; do
  ((count = count + 1))
  echo -n "."
  sleep 1
 done
 echo "."
}

backup_ap() {
 if test ! `df|grep drbd| awk '{print $1}'`; then
  echo "Backing up Webapps & Internal DB"
  check_service drbd drbd stop
  sleep_disp 10
  drbd_drive mount
  tomcat mounted
  drbd_drive umount
  sleep_disp 10
  check_service drbd drbd start
  sleep_disp 10
 else
  echo "Excluding Webapps & Internal DB as DRBD is being used"
 fi
}

backup_sa() {
 echo "Backing up Webapps & Internal DB"
 check_service mon mon.cf stop
 check_service tomcat5 tomcat stop
 sleep_disp 10
 tomcat normal
 check_service tomcat5 tomcat start
 sleep_disp 10
 check_service mon mon.cf start
}

appliance_check() {
 db_used
 if test $APPLIANCE_TYPE = "AA" || test $DR = "yes"; then
  if test $DB_IN_USE = "Internal" || test $DB_IN_USE = "Shipping"; then
   echo "PINsafe DB copied within Tomcat backup"
  else
   mysql
  fi
  tomcat normal
 fi

 if test $APPLIANCE_TYPE = "AP" || test $APPLIANCE_TYPE = "SA" ; then
  if test $DB_IN_USE = "Shipping" || test $DB_IN_USE = "Internal"; then
   if test $APPLIANCE_TYPE = "AP"; then
    backup_ap; fi
   if test $APPLIANCE_TYPE = "SA"; then
    backup_sa; fi
  fi
 fi
}

check_service() {
# Check on Lock file, and either create or remove depending on process status
 if test "`ps -eaf|grep $2|egrep -v "UID|grep|mysqld_safe" 2>/dev/null`"; then
  if test ! -f $LOCK/$1; then
   touch $LOCK/$1; fi
 else
  if test -f $LOCK/$1; then
   rm -f $LOCK/$1; fi
 fi
 
# Check if process is running, and if so set flag to restart it after the backup
 if test $3 = "stop"; then
  if test -f $LOCK/$1; then
   service $1 stop >/dev/null 2>&1
   sleep_disp 5
   if test "`ps -eaf|grep $2|egrep -v "UID|grep|mysqld_safe" 2>/dev/null`"; then
    P_ID=`ps -eaf|grep $2|egrep -v "UID|grep|mysqld_safe" | awk '{print $2}'`
    kill -9 $P_ID
    sleep_disp 5; fi
   serv_state="running"
  else
   serv_state="stopped"
  fi
 fi

 if test $3 = "start"; then
  if test $serv_state; then
   if test $serv_state = "running"; then
    if test $1 = "tomcat5"; then
     service_sands_tomcat
    else
     service $1 start >/dev/null 2>&1
    fi
    serv_state=""
   fi
  fi
 fi
}

service_sands_tomcat() {
 nohup service tomcat5 start > /dev/null 2>&1 &
 echo -n "Tomcat: $1"
}

drbd_drive() {
 if test $1 = "mount"; then
  mkdir -p /mnt/drbd-tmp
  drbd_disk=`grep "/dev/sda" /etc/drbd.conf|tail -1|cut -d\/ -f3|cut -d\; -f1`
  mount /dev/$drbd_disk /mnt/drbd-tmp; fi
 
 if test $1 = "umount"; then
  umount /mnt/drbd-tmp; fi
}

customer_backup() {
 BACKUP_TYPE="A"
 rm -rf /backups/.customer_default/*
 BACKUPROOT=/backups/.customer_default
 NETWORKS=$BACKUPROOT/networking
 audit "Customer default backup"
 create_backup_area tomcat keystore networking
 backup_start Customer
 networking
 misc
 appliance_check
 drbd
 webmin
 mon
 heartbeat
 scripts
 mv /backups/.customer_default.log $BACKUPROOT/default.log
 backup_end
}

factory_backup() {
 BACKUP_TYPE="A"
 if test `ls /backups/.default|head -1`; then
  exit; fi
 BACKUPROOT=/backups/.default
 NETWORKS=$BACKUPROOT/networking
 audit "Factory default backup"
 create_backup_area tomcat keystore networking
 backup_start Factory
 networking
 misc
 appliance_check
 drbd
 webmin
 mon
 heartbeat
 scripts
 mv /backups/.default.log $BACKUPROOT/default.log
 backup_end
}

build_info_file() {
 echo "DATE=`date`" > $BACKUPROOT/$dir_name.info
 echo "TYPE=$BACKUP_TYPE" >> $BACKUPROOT/$dir_name.info
 if test "$backup_reason"; then
  echo "COMMENT=$backup_reason" >> $BACKUPROOT/$dir_name.info; fi
}

backup_all() {
 BACKUP_TYPE="A"
 audit "Full backup initiated"
 create_backup_area tomcat keystore networking
 backup_start
 networking
 misc
 appliance_check
 drbd
 webmin
 mon
 heartbeat
 scripts
 backup_end
 mv $BACKUPROOT.log $BACKUPROOT
 build_info_file
 cp $BACKUPROOT/$dir_name.info /backups
 cd /backups; tar cfz `echo $dir_name`.tar.gz $dir_name
 echo `echo $dir_name`.tar.gz > /backups/latest.txt
 ln -sf /backups/`echo $dir_name`.tar.gz /backups/latest.tar.gz
 rm -rf $BACKUPROOT
}

backup_pinsafe() {
 BACKUP_TYPE="P"
 audit "PINsafe backup initiated"
 create_backup_area tomcat
 backup_start
 appliance_check
 mv $BACKUPROOT.log $BACKUPROOT
 build_info_file
 cp $BACKUPROOT/$dir_name.info /backups
 cd /backups; tar cfz `echo $dir_name`.tar.gz $dir_name
 backup_end
 rm -rf $BACKUPROOT
}

options() {
 echo "backup: error"
 echo "backup: all/pinsafe"
}

initialise

case $1 in
 customer) (customer_backup|tee -a /backups/.customer_default.log) 3>&1 1>&2 2>&3|tee -a /backups/.customer_default.log ; ;;
 factory) (factory_backup|tee -a /backups/.default.log) 3>&1 1>&2 2>&3|tee -a /backups/.default.log ; ;;
 all) (backup_all |tee -a $BACKUPROOT.log) 3>&1 1>&2 2>&3|tee -a $BACKUPROOT.log ; ;;
 pinsafe|pin) (backup_pinsafe |tee -a $BACKUPROOT.log) 3>&1 1>&2 2>&3|tee -a $BACKUPROOT.log ; ;;
 *) options; ;;
esac
