#!/bin/bash
#
# restore - Admin backup restore script
#
# Type  : Utility
#
# Author: Rob Allen
#
# Date  : 29.04.2008
#
# Program Description
# ===================
# Script to restore a Swivel Appliance from a backup.
# Supported scripts: Will only restore from backup.sh
#
#
# Amendment History
# =================
# 29.04.08 Created today (RA)
# 06.05.08 Updated to incorporate paging, and fixed numerous user/error handling
#          issues. Script will not include extra backup archives if they are
#          placed in /backups/restore
# 13.05.08 Updated "warning" routine, to display an appropriate message based on
#          the backup file being restored.
# 02.06.08 Restore lines omitted the DB name to restore! (RA)
# 03.06.08 Included bash_history files in the restore (RA)
# 04.06.08 Fixed bug on chk_tomcat not working, and chown perms on /var/log (RA)
# 11.06.08 Added option to restore just PINsafe DB, excluding config files (RA)
# 29.06.08 Test for bash_history files existing (RA)
# 06.07.08 Added facility to view backup comments (RA)
# 16.07.08 Fixed ownership issue on /var/log/tomcat/catalina.out (RA)
# 02.06.09 Added MYSQL routine to determine version (RA)
# 19.06.09 Added intelligent restore testing, to prevent full backups from one
#          appliance being restored to a different appliance. Check allows
#          full restore to default IPs, providing appliance type matches. (RA)
# 15.08.09 Added option for pinsafe.conf and cmi.conf (RA)
# 25.10.09 Check if any files to restore, if not exit script (RA)
# 25.10.09 Auto detect PINsafe DB name, from backup file for restore (RA)
# 25.10.09 Removed checks on archive when restoring (RA)
# 03.11.09 Updated for 'just' PINsafe DB being available, no other options (RA)
# 14.02.10 Fixed numerous bugs, and removed presumption of using MySQL DB when
#          restoring. Auto detects DB type and act accordingly (RA)
# 17.02.10 Added option to auto respond to SSH connections (RA)
# 09.08.10 Changed restore process, to recover from a single archive (RA)
# 09.08.10 Removed options to restore a network only backup (RA)
# 09.08.10 Simplified backup logic, and removed various bugs (RA)
# 09.09.10 Fixed deletion issue on 'remove_archive' (RA)
# 11.11.10 Placed "" marks around var TYPE to allow spacing (RA)
# 13.12.10 Added check to 'res_mysql' to stop mysql starting, on Single (RA)
# 23.12.10 Altered bulk of script to simplify use, and remove errors (RA)
#

DEBUG=/backups/debug
PINSAFE_CONF=/etc/pinsafe.conf
LOCK=/var/lock/subsys
BACKUPROOT=/backups
AUDIT_LOG=/var/log/cmi/audit.log
restore_vers="0.13"
SSH="ssh -l admin"
NET_SCRIPTS=etc/sysconfig/network-scripts
inv=`tput smso`
nor=`tput rmso`
PINSAFE=/usr/local/tomcat/webapps/pinsafe/WEB-INF
CONFIG_XML=tomcat/webapps/pinsafe/WEB-INF/conf/config.xml

cd $BACKUPROOT

check_debug() {
 if test $DEBUG; then
  if ! test -d $DEBUG; then
   mkdir -p $DEBUG; fi
 fi
}

appliance_type() {
 PINSAFE_TYPE=`grep PINSAFE_TYPE $PINSAFE_CONF|cut -d\= -f2`

 if test $PINSAFE_TYPE = "Primary"; then
  APPLIANCE_TYPE="AA"
  LocalhostIP="`grep Primary_IP /etc/mon/mon.cf|awk '{ print $3}'`"
  RemotehostIP="`grep Standby_IP /etc/mon/mon.cf|awk '{ print $3}'`"
  O_PINSAFE_TYPE="Standby"; fi
 if test $PINSAFE_TYPE = "Standby"; then
  APPLIANCE_TYPE="AA"
  LocalhostIP="`grep Standby_IP /etc/mon/mon.cf|awk '{ print $3}'`"
  RemotehostIP="`grep Primary_IP /etc/mon/mon.cf|awk '{ print $3}'`"
  O_PINSAFE_TYPE="Primary"; fi

 if test "$PINSAFE_TYPE" = "Single"; then
  PINSAFE_TYPE="Single "; fi
 if test "$PINSAFE_TYPE" = "Single "; then
  APPLIANCE_TYPE="SA"
  LocalhostIP="`grep Primary_IP /etc/mon/mon.cf|awk '{ print $3}'`"; fi

 DRBD_USED=`grep DRBD $PINSAFE_CONF|cut -d\= -f2`
 if test $DRBD_USED = "yes"; then
  APPLIANCE_TYPE="AP"; fi
}

initialise() {
 if test -f /$NET_SCRIPTS/ifcfg-eth0; then
  ip=`grep IPADDR /$NET_SCRIPTS/ifcfg-eth0|cut -d\= -f2`; fi

 mkdir -p /backups/restore
 if test $DEBUG; then
  mkdir -p $DEBUG/usr/lib
  mkdir -p $DEBUG/usr/local/tomcat/conf
  mkdir -p $DEBUG/usr/local/tomcat/work
  mkdir -p $DEBUG/root/.ssh
  mkdir -p $DEBUG/home/swivel
  mkdir -p $DEBUG/var/log/tomcat
  mkdir -p $DEBUG/etc/rc.d/init.d
  ln -s $DEBUG/var/log/tomcat $DEBUG/usr/local/tomcat/logs 2>/dev/null
 fi

 if test -f /etc/init.d/mysqld; then
  MYSQL=mysqld
 else
  MYSQL=mysql
 fi
}

audit() {
 audit_date=`date "+%d%m%y %T"`
 echo "$audit_date : $1" >> $AUDIT_LOG
}

heading() {
 clear
 echo "${inv} Swivel Maintenance (c) 2010${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) 2010${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo 
 echo $1
 echo
}

s_heading() {
 clear
 echo "${inv} Swivel Maintenance (c) 2010${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 if test $DR; then
  if test $DR = "yes"; then
   clear
   echo "${inv} Swivel Maintenance (c) 2010${nor}                                                  ${inv}DR${nor}"; fi
 fi
 echo
}

warning() {
 if test $backup_type = "A"; then
  echo "Backup created: $backup_date"
  echo
  echo "   By continuing, this process will overwrite the current PINsafe DB, and"
  echo "   possibly the configuration and appliance settings (including network and"
  echo "   password information). Ensure there is a current backup of the"
  echo "   appliance and PINsafe before continuing."
  echo; fi

 if test $backup_type = "P"; then
  echo "Backup created: $backup_date"
  echo
  echo "   By continuing, this process will overwrite PINsafe on the current"
  echo "   appliance. Ensure there is a current backup of PINsafe before"
  echo "   continuing."
  echo; fi

 if test $backup_type = "P-DB"; then
  echo "Backup created: $backup_date"
  echo
  echo "   By continuing, this process will overwrite the PINsafe DB on the"
  echo "   current appliance. Ensure there is a current backup of PINsafe"
  echo "   before continuing."
  echo; fi

  if test "$backup_text"; then
  echo -n "   ${inv}Reason:${nor} "
  echo $backup_text
  echo; fi
}

yorn() {
 read -p "   Are you sure? (Yes/No): " yn
 check_ans $1 $yn
}

check_ans() {
 if test ! $2; then
  $1; fi
 if test ! $2 = "Yes" && test ! $2 = "yes" && test ! $2 = "y" && test ! $2 = "Y"; then
  $1; fi
}

quit() {
 remove_archive
 echo "Exiting..."
 exit
}

res_etc() {
 if test $backup_type = "A"; then
  ((menu_cnt = menu_cnt + 1))
  echo "      $menu_cnt. Restoring /etc"
  cp -prf $BACKUPROOT/etc $DEBUG/; fi
}

res_misc() {
 if test $backup_type = "A"; then
  ((menu_cnt = menu_cnt + 1))
  echo "      $menu_cnt. Restoring Keystores and SSH"
  cp -pf $BACKUPROOT/keystore/.keystore* $DEBUG/home/swivel
  > $DEBUG/root/.ssh/known_hosts
  cp -prf $BACKUPROOT/root/.ssh $DEBUG/root
  if test -f $BACKUPROOT/root/.bash_history; then
   cp -prf $BACKUPROOT/root/.bash_history $DEBUG/root; fi
  if test -f $BACKUPROOT/home/swivel/.bash_history; then
   cp -prf $BACKUPROOT/home/swivel/.bash_history $DEBUG/home/swivel; fi
 fi
}

full_or_pinsafe_only() {
 s_heading
 if test $ip = $backup_ip || test $ip = "192.168.0.36" || test $ip = "192.168.0.37" || test $ip = "192.168.0.35" && test $PINSAFE_TYPE = $backup_app_type; then
  warning
  echo "1. PINsafe DB only, NO configuration files"
  echo "2. Full restore of appliance, and PINsafe"
  select_q "Backup Menu"
  case $choice in
   1) pinsafe_db_only; exit; ;;
   2) echo; ;;
   `stty intr 
`|0) exit; ;;
   *) s_heading; full_or_pinsafe_only; ;;
  esac
 else
  pinsafe_db_only; exit
 fi
} 

pinsafe_full_or_db_only() {
 s_heading
 warning
 echo "   Select the type of PINsafe restore required"
 echo
 echo "1. PINsafe DB only, NO configuration files"
 echo "2. Full PINsafe recovery, including configuration files"
 select_q "Backup Menu"
 case $choice in
  1) pinsafe_db_only; exit; ;;
  2) echo; ;;
  0) remove_archive; exit; ;;
  *) pinsafe_full_or_db_only; ;;
 esac;
}

pinsafe_db_only() {
 RES_ERR=0
 s_heading
 warning
 yorn quit
 audit "Restore PINsafe DB initiated from: $BACKUPROOT"
 echo
 db_used
 res_pinsafe_db
 if test $RES_ERR = 1; then
  restore_err; fi
 complete
 remove_archive
}

check_running() {
 if test "`ps -eaf|grep $2|egrep -v "UID|grep|mysqld_safe" 2>/dev/null`"; then
  if test ! -f $LOCK/$1; then
   touch $LOCK/$1; fi
 else
  if test -f $LOCK/$1; then
   rm -f $LOCK/$1; fi
 fi

 if test -f $LOCK/$1; then
  value="Running"
 else
  value="Stopped"
 fi
}

service_sands() {
 service $1 $2
 audit "Service $1 : $2"
}

res_mysql() {
 if test $DB_IN_USE = "MySQL" && test $backup_type = "A"; then
  if test -f $BACKUPROOT/mysql/mysql_backup.sql; then
   ((menu_cnt = menu_cnt + 1))
   echo "      $menu_cnt. Restoring MySQL"
   check_running $MYSQL mysqld
   if test $value = "Stopped"; then
    service_sands $MYSQL start 2>/dev/null; fi
   mysql -D mysql < $BACKUPROOT/mysql/mysql_backup.sql
  else
   echo "      $menu_cnt. *** MySQL restore file is missing ***"
  fi
 fi
}

res_pinsafe_db() {
 ((menu_cnt = menu_cnt + 1))

 if test $DB_IN_USE = "MySQL"; then
  res_pinsafe_db_sql; fi
 if test $DB_IN_USE = "Internal"; then
  res_pinsafe_db_derby; fi
 if test $DB_IN_USE = "Shipping"; then
  echo "      $menu_cnt. Shipping DB in use, nothing to restore"; fi
 if test $DB_IN_USE = "External"; then
  echo "      $menu_cnt. External DB in use, manual restore required"; fi
}


res_pinsafe_db_sql() {
 if test ! $value ; then
  check_running $MYSQL mysqld
  if test $value = "Stopped"; then
   service_sands $MYSQL start 2>/dev/null; fi
 fi

 if test -f $BACKUPROOT/mysql/pinsafe_backup.sql; then
  DB_NAME_IN_USE="`grep "Database: " $BACKUPROOT/mysql/pinsafe_backup.sql | cut -d\: -f3 | cut -d\  -f2`"
  echo "      $menu_cnt. Restoring $DB_NAME_IN_USE MySQL DB"

  mysqladmin create $DB_NAME_IN_USE 2> /dev/null
  mysql -D $DB_NAME_IN_USE < $BACKUPROOT/mysql/pinsafe_backup.sql
 fi

 if test $value = "Stopped"; then
  service_sands $MYSQL stop 2>/dev/null; fi
}

res_pinsafe_db_derby() {
 if test $DB_IN_USE = "Internal"; then
  if test -d $BACKUPROOT/tomcat/webapps/pinsafe/WEB-INF/db/swivel; then
   if test -d $DEBUG/$PINSAFE; then
    echo "      $menu_cnt. Restoring PINsafe Internal DB"
    cp -rp $BACKUPROOT/tomcat/webapps/pinsafe/WEB-INF/db $DEBUG/$PINSAFE
   else
    echo "      $menu_cnt. PINsafe is not installed. Unable to restore Internal DB"
    RES_ERR=1
   fi
  else
   echo "      $menu_cnt. Unable to restore PINsafe Internal DB"
   RES_ERR=1
  fi
 fi
}

res_tomcat() {
 if test $backup_type = "A" || test $backup_type = "P" ; then
  ((menu_cnt = menu_cnt + 1))
  if test `ls $BACKUPROOT/tomcat|head -1`; then
   echo "      $menu_cnt. Restoring Tomcat"
   /usr/local/bin/ssh-expect.exp $LocalhostIP >/dev/null 2>&1
   if test $RemotehostIP; then
    /usr/local/bin/ssh-expect.exp $RemotehostIP >/dev/null 2>&1; fi

   check_running heartbeat heartbeat
   if test $value = "Running"; then
    service_sands heartbeat stop 2>/dev/null; fi

   check_running tomcat5 tomcat
   if test $value = "Running"; then
    service_sands tomcat5 stop 2>/dev/null; fi

   if test $APPLIANCE_TYPE = "AP"; then
    if test `df|grep drbd| awk '{print $1}'`; then
     cp -prf $BACKUPROOT/tomcat/webapps $DEBUG/usr/local/tomcat
    else
     echo "*** Unable to restore webapps on this Active/Passive appliance. ***"
    fi
   else
    cp -prf $BACKUPROOT/tomcat/webapps $DEBUG/usr/local/tomcat
   fi
   cp -prf $BACKUPROOT/tomcat/webapps2 $DEBUG/usr/local/tomcat
   cp -prf $BACKUPROOT/tomcat/server.xml $DEBUG/usr/local/tomcat/conf
   cp -prf $BACKUPROOT/tomcat/logs/* $DEBUG/var/log/tomcat
   cp -pf $BACKUPROOT/tomcat/tomcat5 $DEBUG/etc/rc.d/init.d
   rm -rf $DEBUG/usr/local/tomcat/work/*
   chown -R swivel.swivel $DEBUG/var/log/tomcat/*
   chown -R swivel.swivel $DEBUG/usr/local/tomcat/*
  else
   echo "      $menu_cnt. Tomcat is not contained within the archive."
  fi
 fi
}

res_webmin() {
 if test $backup_type = "A" ; then
  ((menu_cnt = menu_cnt + 1))
  echo "      $menu_cnt. Restoring Webmin"
# Webmin already restored by res_etc. This is a placeholder, for information.
 fi
}

res_mon() {
 if test $backup_type = "A" ; then
  ((menu_cnt = menu_cnt + 1))
  echo "      $menu_cnt. Restoring MON"
  cp -prf $BACKUPROOT/usr/lib/mon $DEBUG/usr/lib
 fi
}

res_heartbeat() {
 if test $backup_type = "A" ; then
  ((menu_cnt = menu_cnt + 1))
  echo "      $menu_cnt. Restoring Heartbeat"
# Heartbeat already restored by res_etc. This is a placeholder, for information.
 fi
}

res_drbd() {
 if test $backup_type = "A" && test $APPLIANCE_TYPE = "AP" ; then
  ((menu_cnt = menu_cnt + 1))
  echo "      $menu_cnt. Restoring DRBD"
# DRBD already restored by res_etc. This is a placeholder, for information.
 fi
}

res_scripts() {
 if test $backup_type = "A" ; then
  ((menu_cnt = menu_cnt + 1))
  echo "      $menu_cnt. Restoring scripts"
  cp -prf $BACKUPROOT/usr/local/bin $DEBUG/usr/local
 fi
}

restore_err() {
 echo
 echo "The restore process has failed."
 echo "Please try again, or contact support."
 echo
 read -p "Press Return to Continue." yn
 exit
}

complete(){
 IP=`grep IPADDR /etc/sysconfig/network-scripts/ifcfg-eth0|cut -d\= -f2`
 echo
 echo "The restore process has now completed."
 echo "Please reboot the appliance for all changes to take effect."
 echo "The appliance will respond on ${inv}$IP${nor} after a reboot."
 echo
 read -p "Press Return to Continue." yn
}

create_res_array() {
 for name in `ls -t *.info|grep -v latest`; do
  part1=`echo $name | cut -d\. -f1`
  part2=`echo $name | cut -d\. -f2`
  archive_name="$part1.$part2"
  if test -f $archive_name.tar.gz && test $archive_name.info; then
   echo $archive_name; fi
 done
}

populate_res_array() {
 cd $BACKUPROOT

 archlist=( `create_res_array` )

 count=0
 for name in ${archlist[@]}
 do
  archlist[$count]=$name
  ((count = count + 1))
 done
 totcount=$count
}

disp_backups() {
 dispcount=1
 count=1
 page=1
 for archive_name in ${archlist[@]}
 do
  date_run=`grep ^DATE $archive_name.info |cut -d\= -f2`
  backup_type="`grep ^TYPE $archive_name.info |cut -d\= -f2`"
  if test "$backup_type" = "P"; then
   TYPE_text="PINsafe Backup"; fi
  if test "$backup_type" = "P-DB"; then
   TYPE_text="PINsafe DB Only"; fi
  if test "$backup_type" = "A"; then
   TYPE_text="Full Backup"; fi

  printf "%-3s %-29s %-14s %-15s\n" "$count." "`echo $date_run` :" "$TYPE_text" "($archive_name).tar.gz"
  
  if test $dispcount = 15 && test $count != $totcount; then
   ((page = page + 1))
   dispcount=0
   next_page; fi

  ((dispcount = dispcount + 1))
  ((count = count + 1))
 done

 if test ! $choice; then
  select_q " Exit"
  if test $choice; then
   select_choice
   main; fi
 fi
}

select_q() {
 echo "0. $1"
 echo
 read -p "Select: " choice
}

next_page() {
 select_q "Next Page"

 if test ! $choice; then
  heading "Select a backup archive to restore from (page: $page)."
 else
  if test $choice = 0; then
   heading "Select a backup archive to restore from (page: $page)."
   choice=""
  else
   select_choice
   main
   break
  fi
 fi
}

select_choice() {
 if test $choice -lt 0 || test $choice -gt $totcount; then
  echo "Not a valid option, not that many backups"
  quit; fi
 
 if test $choice -eq 0; then
  echo "Finished"
  quit; fi

 ((choice = choice - 1))
 archive_name=${archlist[$choice]}

}

restore_areas() {
 db_used
 res_etc
 res_mysql
 res_pinsafe_db
 res_tomcat
 res_webmin
 res_mon
 res_heartbeat
 res_drbd
 res_scripts
}

chk_new_archives() {
 cd $BACKUPROOT/restore
 if `ls *.info > /dev/null 2>&1`; then
  s_heading
  echo "Files have been placed in the restore area."
  echo -n "Please wait whilst they are made available for restoration."
  sleep 1
  echo; echo

  for file in `ls *.info`; do
   part1=`echo $file | cut -d\. -f1`
   part2=`echo $file | cut -d\. -f2`
   fname="$part1.$part2"
   cd $BACKUPROOT/restore
   if test -f $fname.info && test -f $fname.tar.gz; then
    mv -f $fname.* $BACKUPROOT
    echo "Restored $fname for use"
   else
    echo "Missing $fname backup archive. Removing orphaned .info file"
    rm -f $fname.info
   fi
  done
  echo; read -p "Press Return to Continue." yn
 fi
}

initialise_backup() {
 backup_text=`grep ^COMMENT $archive_name.info |cut -d\= -f2`
 backup_type="`grep ^TYPE $archive_name.info |cut -d\= -f2`"
 backup_date="`grep ^DATE $archive_name.info |cut -d\= -f2`"

 cd $BACKUPROOT
 tar xfz $archive_name.tar.gz 2> /dev/null 2>&1
 BACKUPROOT=/backups/$archive_name
 if test ! -d $BACKUPROOT; then
  s_heading
  echo "The backup archive has failed to expand."
  echo
  read -p "Press return to exit" yn
  quit; fi

 if test -f $BACKUPROOT/$NET_SCRIPTS/ifcfg-eth0; then
  backup_ip=`grep IPADDR $BACKUPROOT/$NET_SCRIPTS/ifcfg-eth0|cut -d\= -f2`; fi
 if test -f $BACKUPROOT/etc/cmi.conf; then
  backup_app_type=`grep PINSAFE_TYPE $BACKUPROOT/etc/cmi.conf|cut -d\= -f2`; fi
 if test -f $BACKUPROOT/etc/pinsafe.conf; then
  backup_app_type=`grep PINSAFE_TYPE $BACKUPROOT/etc/pinsafe.conf|cut -d\= -f2`
 fi
}

main() {
 initialise_backup

 if test $backup_type = "A" ; then
  full_or_pinsafe_only; fi

 if test $backup_type = "P-DB" ; then
  pinsafe_db_only
  remove_archive
  exit; fi

 if test $backup_type = "P" ; then
  pinsafe_full_or_db_only; fi

 s_heading
 warning

 yorn quit
 audit "Full restore initiated from: $BACKUPROOT"
 s_heading
 warning
 restore_areas
 complete
 remove_archive
}

remove_archive() {
 cd /backups
 if test -d $archive_name; then
  rm -rf $archive_name; fi
}

db_used() {
 CONFIG_XML=$BACKUPROOT/tomcat/webapps/pinsafe/WEB-INF/conf/config.xml
 XML="<option generated=\"true\" selected=\"true\""

 DB_IN_USE=External
 if test -f $CONFIG_XML; then
  if grep "$XML>Shipping</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=Shipping; fi
  if grep "$XML>Internal</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=Internal; fi
  if grep "$XML>MySQL 5</option>" $CONFIG_XML > /dev/null; then
   DB_IN_USE=MySQL; fi
 else
  if test ! $APPLIANCE_TYPE = "AP"; then
   echo "Unable to find the config.xml file."; fi
 fi
}

customer_default() {
 heading "Restore appliance to Customer Defaults"
 BACKUPROOT=/backups/.customer_default
 audit "Restore initiated from: $BACKUPROOT"
 if test -f $LOCK/tomcat5; then
  service tomcat5 stop >/dev/null 2>&1; fi
 backup_type="A"
 cd $BACKUPROOT
 restore_areas
 complete
}

factory_default() {
 heading "Restore appliance to Factory Defaults"
 BACKUPROOT=/backups/.default
 audit "Restore initiated from: $BACKUPROOT"
 if test -f $LOCK/tomcat5; then
  service tomcat5 stop >/dev/null 2>&1; fi
 backup_type="A"
 cd $BACKUPROOT
 restore_areas
 complete
}

check_archive() {
 cd $BACKUPROOT
 if ! `ls *.info > /dev/null 2>&1`; then
  echo "No files available to restore"
  echo; read -p "Press return to exit" yn
  exit; fi
}

check_debug
initialise
appliance_type
if test $1; then
 if test $1 = "factory"; then
  factory_default; exit; fi
 if test $1 = "customer"; then
  customer_default; exit; fi
 fi
chk_new_archives
heading "Select a Backup archive to restore from."
check_archive
populate_res_array
disp_backups
